/***************************************************************************
 * Copyright (c) 2024 Microsoft Corporation 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the MIT License which is available at
 * https://opensource.org/licenses/MIT.
 * 
 * SPDX-License-Identifier: MIT
 **************************************************************************/

/**************************************************************************/
/**                                                                       */
/** USBX Component                                                        */
/**                                                                       */
/**   Device CCID Class                                                   */
/**                                                                       */
/**************************************************************************/
/**************************************************************************/

#define UX_SOURCE_CODE


/* Include necessary system files.  */

#include "ux_api.h"
#include "ux_device_class_ccid.h"
#include "ux_device_stack.h"


/**************************************************************************/
/*                                                                        */
/*  FUNCTION                                               RELEASE        */
/*                                                                        */
/*    _ux_device_class_ccid_uninitialize                  PORTABLE C      */
/*                                                           6.3.0        */
/*  AUTHOR                                                                */
/*                                                                        */
/*    Chaoqiong Xiao, Microsoft Corporation                               */
/*                                                                        */
/*  DESCRIPTION                                                           */
/*                                                                        */
/*    This function uninitialize the USB CCID device.                     */
/*                                                                        */
/*  INPUT                                                                 */
/*                                                                        */
/*    command                               Pointer to ccid command       */
/*                                                                        */
/*  OUTPUT                                                                */
/*                                                                        */
/*    Completion Status                                                   */
/*                                                                        */
/*  CALLS                                                                 */
/*                                                                        */
/*    _ux_utility_memory_free               Free memory                   */
/*    _ux_device_mutex_delete               Delete mutex                  */
/*    _ux_utility_event_flags_delete        Delete event flags            */
/*    _ux_utility_thread_delete             Delete thread                 */
/*                                                                        */
/*  CALLED BY                                                             */
/*                                                                        */
/*    USBX Source Code                                                    */
/*                                                                        */
/*  RELEASE HISTORY                                                       */
/*                                                                        */
/*    DATE              NAME                      DESCRIPTION             */
/*                                                                        */
/*  04-25-2022     Chaoqiong Xiao           Initial Version 6.1.11        */
/*  10-31-2023     Chaoqiong Xiao           Modified comment(s),          */
/*                                            added a new mode to manage  */
/*                                            endpoint buffer in classes, */
/*                                            resulting in version 6.3.0  */
/*                                                                        */
/**************************************************************************/
UINT  _ux_device_class_ccid_uninitialize(UX_SLAVE_CLASS_COMMAND *command)
{

UX_DEVICE_CLASS_CCID                    *ccid;
UX_SLAVE_CLASS                          *ccid_class;
#if !defined(UX_DEVICE_STANDALONE)
UX_DEVICE_CLASS_CCID_RUNNER             *runner;
ULONG                                   i;
#endif

    /* Get the class container.  */
    ccid_class =  command -> ux_slave_class_command_class_ptr;

    /* Get the CCID instance.  */
    ccid = (UX_DEVICE_CLASS_CCID *)ccid_class -> ux_slave_class_instance;

    /* Sanity check.  */
    if (ccid != UX_NULL)
    {

        /* Free allocated resources.  */
#if !defined(UX_DEVICE_STANDALONE)
        _ux_device_thread_delete(&ccid -> ux_device_class_ccid_notify_thread);
        _ux_device_thread_delete(&ccid -> ux_device_class_ccid_thread);
        for (i = 0;
            i < ccid -> ux_device_class_ccid_parameter.ux_device_class_ccid_max_n_busy_slots;
            i ++)
        {
            runner = &ccid -> ux_device_class_ccid_runners[i];
            _ux_device_thread_delete(&runner -> ux_device_class_ccid_runner_thread);
        }
        _ux_device_event_flags_delete(&ccid -> ux_device_class_ccid_events);
        _ux_device_mutex_delete(&ccid -> ux_device_class_ccid_mutex);
        _ux_device_semaphore_delete(&ccid -> ux_device_class_ccid_notify_semaphore);
        _ux_device_mutex_delete(&ccid -> ux_device_class_ccid_response_mutex);
#endif

#if UX_DEVICE_ENDPOINT_BUFFER_OWNER == 1
        /* Free the bulk in endpoint memory.  */
        _ux_utility_memory_free(ccid -> ux_device_class_ccid_endpoint_buffer);
#endif

        /* Free instance memory.  */
        _ux_utility_memory_free(ccid);
    }

    /* Return completion status.  */
    return(UX_SUCCESS);
}
